package fr.lri.keller.td5;

import android.content.Context;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.webkit.JavascriptInterface;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

public class TD5Activity extends AppCompatActivity {

    private final String URL = "https://www.lri.fr/~keller/Enseignement/Orsay/LP-PRISM-AF/TD5.html";

    private WebView webview;
    private EditText editTextUrl;
    private ListView listViewOnglets;

    // Numérotation des onglets. On pourrait être plus informatif en nommant les onglets par les
    // titres des pages web plutôt que par des numéros, mais cela complique un peu
    private int ongletNum = 0;
    private int ongletCourant = ongletNum;

    // Cet objet va permettre d'ajouter dynamiquement des onglets
    ArrayAdapter<String> adaptateur;

    // On va associer la l'historique des urls à chaque onglet
    Map<Integer, Zipper<String>> ongletUrl = new HashMap<Integer, Zipper<String>>();

    // Action à effectuer lorsque l'utilisateur clique sur l'un des onglets
    private AdapterView.OnItemClickListener ongletClickedHandler = new AdapterView.OnItemClickListener() {
        public void onItemClick(AdapterView parent, View v, int position, long id) {
            ongletCourant = position;
            if (ongletCourant == ongletNum + 1) { // L'utilisateur a cliqué sur "Nouveau"
                // On insère un nouvel onglet avec une page blanche
                ongletNum++;
                adaptateur.insert("Onglet " + ongletNum, ongletCourant);
                chargeUrl("about:blank");
            } else {                              // L'utilisateur a cliqué sur un onglet existant
                // On charge l'url en cours associée à cet onglet
                chargeUrl(ongletUrl.get(ongletCourant).current());
            }
        }
    };

    private void chargeUrl(final String url) {
        // Chargement de la page demandée
        // Utiliser directement [webview.loadUrl(url);] fonctionnait avec des versions plus
        // anciennes d'Android, mais n'est plus autorisé sur les APIs récentes : Android impose
        // maintenant que les opérations sur les vues web se fassent uniquement dans le thread
        // principal (ce qui est cohérent avec la politique d'Android), or l'utilisation de code
        // Java depuis JavaScript se fait dans un autre thread (là encore, pour éviter de bloquer
        // l'interface). Pour communiquer avec le thread principal lorsqu'on est dans un autre
        // thread non géré par la classe AsyncTask, on peut utiliser la méthode [post] de la classe
        // [View].
        webview.post(new Runnable() {
            public void run() {
                webview.loadUrl(url);
            }
        });
        // On associe la nouvelle url à l'onglet courant
//        ongletUrl.put(ongletCourant, url);
    }

    private void chargeEditTextUrl() {
        // Récupération de l'url entrée par l'utilisateur
        String url = editTextUrl.getText().toString();
        // Chargement de la page demandée
        chargeUrl(url);
        // Nettoyage de la barre d'url
        editTextUrl.setText("");
    }

    public class WebAppInterface {
        Context mContext;

        // Constructeur, récupérant le contexte de l'activité
        WebAppInterface(Context c) {
            mContext = c;
        }

        // Affiche un petit message
        @JavascriptInterface
        public void alert(String toast) {
            Toast.makeText(mContext, toast, Toast.LENGTH_SHORT).show();
        }

        // Charge l'url de la zone de texte
        @JavascriptInterface
        public void loadUserUrl() {
            chargeEditTextUrl();
        }
    }

    // On fait une nouvelle classe héritant de WebViewClient pour effectuer des actions
    // supplémentaires : lorsque l'utilisateur navigue sur le web en cliquant sur des liens, on veut
    // se souvenir que c'est le nouveau lien qui est associé à l'onglet courant
    private class WebViewClientOnglet extends WebViewClient {
        @Override
        public void onPageFinished (WebView view, String url) {
            // On fait l'action de base
            super.onPageFinished(view, url);
            // On ajoute la nouvelle url à l'historique de l'onglet courant si ce n'est pas déjà là
            // où on en est dans l'historique
            Zipper<String> historique = ongletUrl.get(ongletCourant);
            if (historique == null) { // C'est un nouvel onglet
                // Il faut faire un nouvel historique
                historique = new Zipper<String>();
                historique.add(url);
                ongletUrl.put(ongletCourant, historique);
            } else {                  // C'est un onglet déjà existant
                // On ajoute la nouvelle url à l'historique (effet de bord) si ce n'est pas déjà là
                // où on en est (afin de gérer les boutons "PRÉCÉDENT" et "SUIVANT")
                if (!url.equals(historique.current())) {
                    historique.add(url);
                }
            }
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_td5);
        // Récupération des vues
        webview = (WebView) findViewById(R.id.webview);
        editTextUrl = (EditText) findViewById(R.id.editTextUrl);
        listViewOnglets = (ListView) findViewById(R.id.listViewOnglets);
    }

    @Override
    protected void onStart() {
        super.onStart();
        // Permet d'avoir quelque chose qui se rapproche plus d'un navigateur, et donc d'ouvrir des
        // liens, par exemple
        webview.setWebViewClient(new WebViewClientOnglet());
        // Activation de JavaScript (qui est désactivé par défaut)
        WebSettings webSettings = webview.getSettings();
        webSettings.setJavaScriptEnabled(true);
        // Lien entre la classe WebAppInterface et le code JavaScript qui tourne dans l'application web
        webview.addJavascriptInterface(new WebAppInterface(this), "Android");
        // On attache un adaptateur à la liste des onglets
        adaptateur = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1);
        listViewOnglets.setAdapter(adaptateur);
        // On a un premier onglet avec notre page web actuelle, et de quoi ouvrir un nouvel onglet
        adaptateur.add("Onglet " + ongletNum);
        adaptateur.add("Nouveau");
        // On fait réagir la liste des onglets aux clics de l'utilisateur
        listViewOnglets.setOnItemClickListener(ongletClickedHandler);
        // Chargement de la page web voulue
        chargeUrl(URL);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_td5, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    // Les boutons "PRÉCÉDENT" et "SUIVANT" doivent maintenant tenir compte des historiques des
    // différents onglets
    public void precedent(View view) {
        Zipper<String> historique = ongletUrl.get(ongletCourant);
        String url = historique.previous();
        if (url != null) {
            chargeUrl(url);
        }
    }

    public void suivant(View view) {
        Zipper<String> historique = ongletUrl.get(ongletCourant);
        String url = historique.next();
        if (url != null) {
            chargeUrl(url);
        }
    }

    public void rafraichir(View view) {
        webview.reload();
    }

    public void charger(View view) {
        chargeEditTextUrl();
    }
}
