package fr.lri.keller.td5;

/**
 * Un zipper est une liste sur laquelle se trouve un curseur qui peutse déplacer à gauche et à
 * droite. Cette structure de données est très utile pour représenter un historique dans lequel on
 * navigue.
 *
 * On peut l'implémenter à l'aide de deux listes usuelles, une représentant la partie de la liste
 * qui se trouve avant le curseur (inversée), et l'autre la partie de la liste qui se trouve après
 * le curseur.
 */

import java.util.LinkedList;
import java.util.NoSuchElementException;

public class Zipper<E> {

    private LinkedList<E> before, after;

    public Zipper() {
        before = new LinkedList<E>();
        after = new LinkedList<E>();
    }

    public E current() {
        return after.getFirst();
    }

    public E previous() {
        try {
            E e = after.pop();
            before.push(e);
            return current();
        } catch (NoSuchElementException e) {
            return null;
        }
    }

    public E next() {
        try {
            E e = before.pop();
            after.push(e);
            return e;
        } catch (NoSuchElementException e) {
            return null;
        }
    }

    public void add(E e) {
        after.push(e);
        before = new LinkedList<E>();
    }

}